///////////////////////////////////////////////////////////////////////////////
// Author: Dave Toth
// Date: Summer/Fall 2005
//
// This Service records when the screen saver of a computer running Windows
// is active.  In addition to my own code, I adapted code from the following
// sources:
//
// To get the MAC Address of the computer:
// from TimothyRobertHeider
// http://forum.java.sun.com/thread.jspa?forumID=31&threadID=421257
//
// To do Windows socket programming:
// http://tangentsoft.net/wskfaq/examples/basics/main.cpp
// http://tangentsoft.net/wskfaq/examples/basics/ws-util.cpp
//
// For spawning threads and determining whether the screensaver is active:
// MSDN
//
// Instructions for how to write a Windows Service that I used:
// http://www.devsource.com/article2/0,1759,1713349,00.asp?
// kc=DSNKT0403KTX1K0000642
//
// The service name that shows up in the ServiceManagement Console is CccIG.
// The image name that shows up in the TaskManager is cccig.exe.
///////////////////////////////////////////////////////////////////////////////

#pragma once

using namespace System;
using namespace System::Collections;
using namespace System::ServiceProcess;
using namespace System::ComponentModel;


///////////////////////////////////////////////////////////////////////////////
// Function Prototypes                                                       //
///////////////////////////////////////////////////////////////////////////////

// Function that the thread we spawn to collect data runs.  This is where 
// the data collection takes place.
DWORD WINAPI DataCollection(LPVOID lpParam); 

// Function that the thread we spawn to do the data transfer runs.
DWORD WINAPI DataTransfer(LPVOID lpParam);

// Function that the thread we spawn to do the synchronization.
DWORD WINAPI SynchronizeFiles(LPVOID lpParam);

// Called if the program hits a fatal error.
void Halt();

// Writes the MAC address of the computer to the output file.
BOOL GetMacAddress(); 

// Sends a file that is ready to the server.
int SendFile(char *sourceFile);

// Returns the address of the hostname passed in.
u_long LookupAddress(const char* pcHost); 

// Makes a socket connection to the server and returns the socket.
SOCKET EstablishConnection(u_long nRemoteAddr, u_short nPort); 

// Reads the response from the server and ensures it's ok.
int ReadReply(SOCKET sd); 

// Sends the data from the file to the server.
bool SendData(SOCKET sd, char *sourceFile); 


///////////////////////////////////////////////////////////////////////////////
// Global Variables                                                          //
///////////////////////////////////////////////////////////////////////////////

// The type of computer.
//char *computerType = "PUBLIC";
//char *computerType = "OFFICE";
//char *computerType = "HOME";
char *computerType = "STUDENT";

// The file we store the start time of the experiment in.
FILE *dataFile;

// The data file we use to store the information.
FILE *outputFile; 

// The file where we keep track of the state of each data file.
FILE *sentFile;

// An array to hold the status of each file that must be sent to the server.
// Fill it with 0 = NOT_READY.
char fileStatus[28] = {NOT_READY, NOT_READY, NOT_READY, NOT_READY, NOT_READY,
NOT_READY, NOT_READY, NOT_READY, NOT_READY, NOT_READY, NOT_READY, NOT_READY, 
NOT_READY, NOT_READY, NOT_READY, NOT_READY, NOT_READY, NOT_READY, NOT_READY, 
NOT_READY, NOT_READY, NOT_READY, NOT_READY, NOT_READY, NOT_READY, NOT_READY, 
NOT_READY, NOT_READY};

// The number of data files that have been sent to the server.
int sentResults = 0;

// The buffer we use to make the file name for our output files.
char fileName[256];

// The buffer we use to make the file name for sending files.
char sendFileName[256];

// A buffer to hold the MAC Address
char macAddress[18];

// A buffer to hold the start and stop times.
char startAndStop[100];

// A buffer to hold data we read in from a file.
char dataReadIn[100];

// The variable we use to store the result of functions.
int result; 

// The last time a sample was taken of the screen saver status.
time_t currentTime; 

// The time the experiment should end.
long endTime; 

// Handle to the thread we spawn to collect data.
HANDLE hThread; 

// The ID of the thread we spawn to collect data.
DWORD dwThreadId; 

// Handle to the thread we spawn to transmit data.
HANDLE hThread2; 

// The ID of the thread we spawn to transmit data.
DWORD dwThreadId2; 

// Handle to the thread we spawn to synchronize files.
HANDLE hThread3; 

// The ID of the thread we spawn to synchronize files.
DWORD dwThreadId3; 

// Return value of the call to determine whether the screen saver is running.
BOOL result1;   

// Indicates whether the first thread has stopped.
BOOL thread1Running;

// Indicates whether the second thread has stopped.
BOOL thread2Running = FALSE;

// Indicates whether the third thread has stopped.
BOOL thread3Running;

// Indicates whether the threads should keep running.  Once the service is 
// stopped, they should not and this gets set to FALSE.
BOOL keepRunning = TRUE;

// Parameter for the call to determine whether the screen saver is running.
UINT uiParam;  

// Parameter for the call to determine whether the screen saver is running.
UINT fWinIni;  

// Parameter for the call to determine whether the screen saver is running.
// Indicates whether the screen saver is running.
BOOL realResult;

// The information for the network cards.  Assume there are no more than 3 
// in a computer.
IP_ADAPTER_INFO AdapterInfo[3]; 

// Length of the buffer size for NIC info.
DWORD dwBufLen;

// Holds the return value of the call to GetAdaptersInfo().
DWORD dwStatus ;

// Pointer to an IP_ADAPTER_INFO struct for getting the MAC Address.
PIP_ADAPTER_INFO pAdapterInfo;

// Indicates whether the synchronization thread needs to rewrite the 
// file that keeps track of the state of files.
BOOL filesChanged = FALSE;

// The state of the screensaver ON, OFF, or UNKNOWN the last time it was 
// sampled.
int state = UNKNOWN;



namespace cccig
{
	/// <summary> 
	/// Summary for cccigWinService
	/// </summary>
	///
	/// WARNING: If you change the name of this class, you will need to change
	///          the 'Resource File Name' property for the managed resource 
	///          compiler tool associated with all .resx files this class 
	///          depends on.  Otherwise, the designers will not be able to 
	///          interact properly with localized resources associated with
	///          this form.
	public __gc class cccigWinService : public System::ServiceProcess::
		ServiceBase 
	{
	public:
		cccigWinService()
		{
			InitializeComponent();    
		}
		/// <summary>
		/// Clean up any resources being used.
		/// </summary>
		void Dispose(bool disposing)
		{
			if (disposing && components)
			{
				components->Dispose();
			}
			__super::Dispose(disposing);
		}
		
	protected:
		/// <summary>
		/// Set things in motion so your service can do its work.
		/// </summary>
		void OnStart(String* args[])
		{
			// TODO: Add code here to start your service.

			keepRunning = TRUE;
			state = UNKNOWN;

			// Fill the MAC Address buffer.  We need this to have the 
			// experiment work.
			if (GetMacAddress() == FALSE) // We can't recover from this.
			{
				Halt();
				return;
			}

			// Open the file that keeps track of what files are ready to send,
			// not ready, or successfully sent and set the array from it.
			sentFile = NULL;
			sentFile = fopen("C:\\ccxmt", "r");
			if (sentFile != NULL) // The file exists.
			{
				// Read in all the values for the array.
				for (int temp = 0; temp < 28; temp ++)
				{
					int c = fgetc(sentFile);
					if (((char) c) == '0')
					{
						fileStatus[temp] = NOT_READY;
					}
					else 
					{
						if (((char) c) == '1')
						{
							fileStatus[temp] = READY_TO_SEND;
						}
						else
						{
							fileStatus[temp] = SENT_OK;
							sentResults ++;
						}
					}
				}
				fclose(sentFile);

				// If all the files were sent successfully, exit.
				if (sentResults == 28)
				{
					return;
				}
			}
			else
			{
				// The file does not exist.  Create it with all files marked as
				// not ready.  The array is set to NOT_READY by default so no 
				// need to reset that.
				sentFile = fopen("C:\\ccxmt", "w");
				if (sentFile == NULL) // If there's an error we can't recover.
				{
					Halt();
					return;
				}
				fprintf(sentFile, "0000000000000000000000000000");
				fflush(sentFile);
				fclose(sentFile);
			}

			// Now start the data collection thread, the data transmission
			// thread, and the file synchronization thread.  They'll do all
			// the work from here on out.
			hThread = CreateThread(NULL, 0, DataCollection, NULL, 0, 
				&dwThreadId);  

			hThread2 = CreateThread(NULL, 0, DataTransfer, NULL, 0, 
				&dwThreadId2);  

			hThread3 = CreateThread(NULL, 0, SynchronizeFiles, NULL, 0,
				&dwThreadId3);
		}
		
		/// <summary>
		/// Stop this service.
		/// </summary>
		void OnStop()
		{     
			// TODO: Add code here to perform any tear-down necessary to stop 
			// your service.

			keepRunning = FALSE;

			while ((thread1Running == TRUE) || (thread2Running == TRUE) || (thread3Running == TRUE)) 
			{
				Sleep(1000);
			}

			// Just close the output file.  Nothing else should get done here.
			if (outputFile != NULL)
			{
				// Flush anything being written.
				fflush(outputFile);
				// Close the file.
				fclose(outputFile);
			}
			state = UNKNOWN;
		}
			
	private:
		/// <summary>
		/// Required designer variable.
		/// </summary>
		System::ComponentModel::Container *components;

		/// <summary> 
		/// Required method for Designer support - do not modify 
		/// the contents of this method with the code editor.
		/// </summary>	
		void InitializeComponent(void)
		{
			// 
			// cccigWinService
			// 
			this->CanPauseAndContinue = true;
			this->ServiceName = S"cccigWinService";

		}		
	};
}


/////////////////////////////////////////////////////////
// Function that the file synchronization thread runs. //
/////////////////////////////////////////////////////////
DWORD WINAPI SynchronizeFiles(LPVOID lpParam)
{
	thread1Running = TRUE;
	sentFile = NULL;
	sentFile = fopen("C:\\ccxmt", "r+");
	if (sentFile == NULL) // The file doesn't exist.  Halt.
	{
		Halt(); // We can't recover from this.
		thread1Running = FALSE;
		return -1;
	}

	int writeResult;
	while ((sentResults < 28) && (keepRunning == TRUE))
	{
		if (filesChanged == TRUE)
		{
			// Write success to the file that keeps track of 
			// successfully sent files.

			// Move to the beginning of the file.
			rewind(sentFile);

			// Now write out the array.
			if ((writeResult = fprintf(sentFile, fileStatus)) > 0)
			{
				writeResult = fflush(sentFile);
				if (writeResult != 0) // If the flush failed
				{
					// This is an unrecoverable error.  Give up.
					Halt();
					thread1Running = FALSE;
					sentResults = 99;
				}
			}
			else 
			{
				Halt();
				thread1Running = FALSE;
				sentResults = 99;
			}
			filesChanged = FALSE;			
		}
		Sleep(1000);
	}

	thread1Running = FALSE;
	return 0;
}


/////////////////////////////////////////////////////////////////
// Function that the thread we spawn to collect the data runs. //
/////////////////////////////////////////////////////////////////
DWORD WINAPI DataCollection( LPVOID lpParam ) 
{
	thread2Running = TRUE;

	int iterations = 0;

	// First thing we need to do is figure out where we are in the experiment.
	// That really means which of the 28 days are we on. 
	
	// Declare a couple variables we need for this.
	div_t divResult;
	int fileIndex;
	long startTime;
	
	// We open the file we created when we started the experiment.  If it 
	// doesn't exist, then we've just started now.
	dataFile = NULL;
	dataFile = fopen("C:\\cccdat", "r");
	if (dataFile == NULL) // The file does not exist so start the experiment.
	{
		// Create the file we need here and put in the current time.
		dataFile = fopen("C:\\cccdat", "w");
		if (dataFile == NULL)
		{
			Halt(); // We can't recover from this.
			thread2Running = FALSE;
			return -1;
		}
		
		// Get the current date and write it to the file.
		time(&currentTime);
		startTime = currentTime;
		result = fprintf(dataFile, "%ld\n", currentTime);
		if (result < 1) // If zero bytes were written or there was an error
		{
			Halt(); // We can't recover from this kind of error.
			thread2Running = FALSE;
			return -1;
		}
		result = fflush(dataFile);
		if (result != 0) // If the flush failed
		{
			Halt(); // We can't recover from this kind of error.
			thread2Running = FALSE;
			return -1;
		}
		fclose(dataFile);
		endTime = currentTime + EXPERIMENT_TIME;
	}
	else // The file exists.  Read the time the experiment started from it.
	{
		// Get the current date.
		time(&currentTime);

		if (fgets(dataReadIn, 98, dataFile) == NULL) // If there's an error
		{
			Halt(); // We can't recover from this.
			thread2Running = FALSE;
			return -1;
		}
		fclose(dataFile);
		startTime = atoi(dataReadIn);
		endTime = startTime + EXPERIMENT_TIME;
	}

	if (sprintf(startAndStop, "%ld_%ld", startTime, endTime) < 1)
	{
		// We can't recover from this.
		Halt();
		thread2Running = FALSE;
		return -1;
	}

	// So at this point, the dataFile exists with the time the experiment
	// was started printed in it.  We have calculated when the experiment
	// should stop.  Now we move forward, by sampling and placing the
	// result in the proper file.

	// First determine the proper file to write to.
	divResult = div((currentTime - startTime), ONE_DAY);
	fileIndex = divResult.quot;
	if (fileIndex > 27) 
	{
		fileIndex = 27;
	}
	sprintf(fileName, "C:\\cccdat%d", fileIndex);

	// MARK ALL FILES BEFORE THIS IN THE SEQUENCE AS READY TO SEND AND
	// SET THE CHANGED FLAG TO TRUE - OTHERWISE A FILE WHOSE TIME WAS UP
	// WHILE THE COMPUTER WAS POWERED DOWN WILL NEVER BE SENT
	for (int loop = 0; loop < fileIndex; loop ++)
	{
		if (fileStatus[loop] == NOT_READY)
		{
			FILE *tempFile = NULL;
			char tempName[256];
			sprintf(tempName, "C:\\cccdat%d", loop);
			tempFile = fopen(tempName, "a+");
			
			if (tempFile == NULL) 
			{
				Halt(); // We can't recover from this.
				thread2Running = FALSE;
				return -1;
			}
			fclose(tempFile);
			fileStatus[loop] = READY_TO_SEND;
			filesChanged = TRUE;
		}
	}

	// Now loop until it's time to open the next file
	do {
		// Write to the appropriate file.

		// Open the file for appending.  If that fails, exit.
		outputFile = fopen(fileName, "a");
		if (outputFile == NULL)
		{
			Halt(); // We can't recover from this.
			thread2Running = FALSE;
			return -1;
		}

		while ((currentTime < (startTime + ((1 + fileIndex) * ONE_DAY))) && (keepRunning == TRUE))
		{
			if (iterations == 0) 
			{
				// Sample the screen saver.
				result1 = SystemParametersInfoA(SPI_GETSCREENSAVERRUNNING, uiParam,
					&realResult, fWinIni);
				// Get the time we did the sampling.  We also use it for the 
				// condition of the while loop to see when the experiment is over.
				time(&currentTime); 
				// Write to the file.
				
				if (state == UNKNOWN)
				{
					// Print timestamp.
					result = fprintf(outputFile, "\n%ld\n", currentTime);
					if (result < 1)
					{
						Halt();
						thread2Running = FALSE;
						return -1;
					}
					result = fflush(outputFile);
					if (result != 0)
					{
						Halt();
						thread2Running = FALSE;
						return -1;
					}
	
					if (realResult == TRUE)
					{
						state = ON;
					}
					else 
					{
						state = OFF;
					}
				}
				else if (state == ON)
				{
					if (realResult == TRUE)
					{
						// Print *
						result = fprintf(outputFile, "*");
						if (result < 1)
						{
							Halt();
							thread2Running = FALSE;
							return -1;
						}
						result = fflush(outputFile);
						if (result != 0)
						{
							Halt();
							thread2Running = FALSE;
							return -1;
						}
					}
					else 
					{
						// Print timestamp.
						result = fprintf(outputFile, "\n%ld\n", currentTime);
						if (result < 1)
						{
							Halt();
							thread2Running = FALSE;
							return -1;
						}
						result = fflush(outputFile);
						if (result != 0)
						{
							Halt();
							thread2Running = FALSE;
							return -1;
						}

						state = OFF;
					}
				}
				else if (state == OFF) // Must be OFF if not UNKNOWN or ON
				{
					if (realResult == FALSE)
					{
						// Print @
						result = fprintf(outputFile, "@");
						if (result < 1)
						{
							Halt();
							thread2Running = FALSE;
							return -1;
						}
						result = fflush(outputFile);
						if (result != 0)
						{
							Halt();
							thread2Running = FALSE;
							return -1;
						}
					}
					else 
					{
						// Print timestamp.
						result = fprintf(outputFile, "\n%ld\n", currentTime);
						if (result < 1)
						{
							Halt();
							thread2Running = FALSE;
							return -1;
						}
						result = fflush(outputFile);
						if (result != 0)
						{
							Halt();
							thread2Running = FALSE;
							return -1;
						}

						state = ON;
					}
				}
			}	
			iterations ++;
			iterations = iterations % 10;
			// Sleep for 1 second.
			Sleep(1000);
		}
		// Now we close the file.
		fclose(outputFile);

		// Update the array.
		fileStatus[fileIndex] = READY_TO_SEND;
		filesChanged = TRUE;
		fileIndex ++;
		sprintf(fileName, "C:\\cccdat%d", fileIndex);
	} while ((currentTime < endTime) && (keepRunning == TRUE));

	thread2Running = FALSE;
	return 0;
}



//////////////////////////////////////////////////////////////////
// Function that the thread we spawn to transmit the data runs. //
//////////////////////////////////////////////////////////////////
DWORD WINAPI DataTransfer(LPVOID lpParam) 
{
	thread3Running = TRUE;
	int ticksTilSend = /*60;//*/ 300;
	// Until we've sent all the data files...
	while ((sentResults < 28) && (keepRunning == TRUE))
	{
		if (ticksTilSend <= 0)
		{
			ticksTilSend = /*60;//*/ 300;
			// This loop sends all available data files.
			for (int index = 0; index < 28; index ++)
			{
				// If a file is ready to send, send it and mark it sent if done.
				if (fileStatus[index] == READY_TO_SEND)
				{
					// Try to send the file
					// Build the file name.
					sprintf(sendFileName, "C:\\cccdat%d", index);

					if (SendFile(sendFileName) == SUCCESS)
					{
						// Mark the files sent successfully in the array
						fileStatus[index] = SENT_OK;
						filesChanged = TRUE;

						sentResults ++;

						// Delete the data file that we sent.
						remove(sendFileName);
					}
				}
			}
		}
		ticksTilSend --;
		Sleep(1000); // Now sleep for 1 second.
	}

	// That's the end of this task or the service was stopped.
	// Close the file where we keep track of the sent files.
	fclose(sentFile);

	thread3Running = FALSE;
	return 0;
}


///////////////////////////////////////////////////////////////////////
// This function writes the MAC Address of the computer to a buffer. //
///////////////////////////////////////////////////////////////////////
BOOL GetMacAddress()
{
	dwBufLen = sizeof(AdapterInfo); // Save memory size of buffer
	dwStatus = GetAdaptersInfo( // Call GetAdapterInfo
	AdapterInfo, // [out] buffer to receive data
	&dwBufLen); // [in] size of receive data buffer

	if(dwStatus == ERROR_SUCCESS) 
	{
		pAdapterInfo = AdapterInfo; // Contains pointer to current adapter info
		int x1 = pAdapterInfo->Address[0];
		int x2 = pAdapterInfo->Address[1];
		int x3 = pAdapterInfo->Address[2];
		int x4 = pAdapterInfo->Address[3];
		int x5 = pAdapterInfo->Address[4];
		int x6 = pAdapterInfo->Address[5];

		result = sprintf(macAddress, "%0.2x_%0.2x_%0.2x_%0.2x_%0.2x_%0.2x",
		x1, x2, x3, x4, x5, x6);
		if (result == -1)
		{
			return FALSE;
		}
	}
	else 
	{
		return FALSE;
	}
	return TRUE;
}



///////////////////////////////////////////////////////////////
// Sends the data in the files that are ready to the server. //
///////////////////////////////////////////////////////////////
int SendFile(char *sourceFile)
{
	// Establish a connection to the server.
	// Start Winsock.
	WSAData wsaData;
	int nCode;
	if ((nCode = WSAStartup(MAKEWORD(1, 1), &wsaData)) != 0) {
	    return FAILURE;
	}

    // Find the server's address
    u_long nRemoteAddress = LookupAddress("FIRE-STORM.ind.wpi.edu");
    if (nRemoteAddress == INADDR_NONE) 
	{
        return FAILURE;
    }
    in_addr Address;
    memcpy(&Address, &nRemoteAddress, sizeof(u_long)); 

    // Connect to the server
    SOCKET sd = EstablishConnection(nRemoteAddress, htons(19501));
    if (sd == INVALID_SOCKET) {
        return FAILURE;
    }

    // Send the file to the server.
    int nBytes;

    if (SendData(sd, sourceFile)) 
	{
		if ((nBytes = ReadReply(sd)) > 0) 
		{
			if (nBytes > 20) 
			{
				return FAILURE;
			}
		}
		else if (nBytes == 0) 
		{
			return FAILURE;
		}
		else 
		{
			return FAILURE;
		}
	}
	else 
	{
        return FAILURE;
    }

	// Shut Winsock off.
	WSACleanup();

	return SUCCESS;
}



/////////////////////////////////////////////////////////////////////////
// Given an address string, determine if it's a dotted-quad IP address //
// or a domain address.  If the latter, ask DNS to resolve it.  In     //
// either case, return resolved IP address.  If we fail, we return     //
// INADDR_NONE.                                                        //
/////////////////////////////////////////////////////////////////////////
u_long LookupAddress(const char* pcHost)
{
	// For BTI deployment, comment out the method contents and replace 
	// with just the following 3 lines:
	/*
	char* hostAddress = "192.168.1.26";
	u_long nRemoteAddr = inet_addr(hostAddress);
	return nRemoteAddr;
	*/
	
	// For non-BTI deployment, comment out the lines above this and
	// uncomment the lines beneath this.
		
    u_long nRemoteAddr = inet_addr(pcHost);
    if (nRemoteAddr == INADDR_NONE) {
        // pcHost isn't a dotted IP, so resolve it through DNS
        hostent* pHE = gethostbyname(pcHost);
        if (pHE == 0) {
            return INADDR_NONE;
        }
        nRemoteAddr = *((u_long*)pHE->h_addr_list[0]);
    }
    return nRemoteAddr;
	
}


/////////////////////////////////////////////////////////////////////////////
// Connects to a given address, on a given port, both of which must be     //
// in network byte order.  Returns newly-connected socket if we succeed,   //
// or INVALID_SOCKET if we fail.                                           //
/////////////////////////////////////////////////////////////////////////////
SOCKET EstablishConnection(u_long nRemoteAddr, u_short nPort)
{
    // Create a stream socket
    SOCKET sd = socket(AF_INET, SOCK_STREAM, 0);
    if (sd != INVALID_SOCKET) 
	{
        sockaddr_in sinRemote;
        sinRemote.sin_family = AF_INET;
        sinRemote.sin_addr.s_addr = nRemoteAddr;
        sinRemote.sin_port = nPort;
        if (connect(sd, (sockaddr*)&sinRemote, sizeof(sockaddr_in)) == 
			SOCKET_ERROR) 
		{
            sd = INVALID_SOCKET;
        }
    }
    return sd;
}


//////////////////////////////////////////////////////////////////////////////
// Sends the data to the server.  Returns true on success, false otherwise. //
//////////////////////////////////////////////////////////////////////////////
bool SendData(SOCKET sd, char *sourceFile)
{
	FILE *sourceData;
	if ((sourceData = fopen(sourceFile, "r")) == NULL)
	{
		return false;
	}

	char fullFileName[100];
	sprintf(fullFileName, "%s_%s_", macAddress, sourceFile);

	// Send the full file name = MAC Address_name of the file.
	if (send(sd, fullFileName, (2 + strlen(macAddress) + strlen(sourceFile)), 
		0) == SOCKET_ERROR) 
	{
		return false;
	}

	// Send the time the experiment started and stopped.
	if (send(sd, startAndStop, strlen(startAndStop), 0) == SOCKET_ERROR) 
	{
		return false;
	}

	// Send a line break.
	if (send(sd, "\n", strlen("\n"), 0) == SOCKET_ERROR) 
	{
		return false;
	}


	// Send the contents of the file over the socket to the server.
	while (fgets(dataReadIn, 99, sourceData) != NULL) 
	{
		//printf("Read In: %s", dataReadIn);
		if (send(sd, dataReadIn, strlen(dataReadIn), 0) == SOCKET_ERROR) 
		{
			return false;
		}
	}
	fclose(sourceData);

	if (send(sd, "\nDONE\n", strlen("\nDONE\n"), 0) == SOCKET_ERROR) 
	{
		return false;
	}
    return true;
}


//////////////////////////////////////////////////////////////////////////
// Read the reply from the server and ensure success.  Returns -1 on    //
// error, 0 on connection closed, SUCCESS on success, UNKNOWN_REPLY on  //
// successfull read that returned something other than "OK  ".          //
//////////////////////////////////////////////////////////////////////////
int ReadReply(SOCKET sd)
{
	// Read reply from server
    char acReadBuffer[5];
    int nNewBytes = recv(sd, acReadBuffer, 5, 0);
        
	if (nNewBytes == SOCKET_ERROR) 
	{
		return -1;
    }
    else if (nNewBytes == 0) 
	{
        return 0;
    }
    // Check data for sanity
    if (strncmp(acReadBuffer, "OK  ", 5) == 0) 
	{
		return SUCCESS;
	}
    return UNKNOWN_REPLY;
}

/////////////////////////////////////////////////////////////////////////
// This is called when we hit an error we can't recover from.  It just //
// halts the program in a known state and gives up.                    //
/////////////////////////////////////////////////////////////////////////
void Halt()
{
	// Just create a file to signal there was a fatal error.
	FILE *errorFile;
	errorFile = fopen("C:\\cccerr", "w");
	fclose(errorFile);
}